/**
 * Nokia 1202/1203/1280 LCD display library for PIC16fx86
 *
 * :  
 *    LPC: Ivan A-R <ivan@tuxotronic.org>
 *
 *  LCD: STE2007.
 * 96 x 68 pixels.
 * 1.3" 27.5 x 19.5 mm active area.
 *    8 ,  1   .
 *   EEPROM 93C86  .
 */

#include "config.h"
#include "lcd1202.h"
#ifndef USE_EEPROM
#include "font.h"
#endif

void lcd_init_gpio(void);
void lcd_reset(void);
static void lcd_ee_write_byte(unsigned char data, unsigned char rc, unsigned char ee_data, unsigned char ee_b8);

struct {
    unsigned char inverse:1;
} flags;


/* hw undepended functions */

void lcd_init(void)
{
    lcd_init_gpio();
    lcd_reset();

    __delay_ms(10);
    lcd_write_byte(0xA4, 0); // Power saver off
    lcd_write_byte(0x2F, 0); // Power control set
    lcd_write_byte(0xAF, 0); // LCD display on

    lcd_clr();

    flags.inverse = 1;
}

void lcd_clr(void)
{
    lcd_home();
    // i  LCD
    unsigned int i;
    for(i=16*6*9; i>0; i--)
    {
        lcd_write_byte(0, 1);
    }
}

//   row,  col
void lcd_pos(unsigned char row, unsigned char col)
{
    lcd_set_row(row);
    lcd_set_col(col*FONT_WIDTH);
}

// 
void lcd_char(unsigned char c)
{
#ifdef USE_EEPROM
        eeprom_to_lcd(c*FONT_WIDTH, FONT_WIDTH);
#else
	unsigned char line;
	unsigned char ch;

        unsigned char* f = &(font6_8[c-32]);

	for(line=0;line<FONT_WIDTH;line++)
	{
		ch = *f++;
        if(flags.inverse) ch = ch ^ 0xFF;
		lcd_write_byte(ch, 1);
	}
#endif
}

// 
void lcd_str(char* str)
{
    unsigned char c;
    while((c = *str) != 0)
    {
        lcd_char(c);
        str++;
    }
}

// 
void lcd_inverse(unsigned char inv)
{
    flags.inverse = inv;
    if(inv)
        lcd_write_byte(0xA7, 0); // reverse display
    else
        lcd_write_byte(0xA6, 0); // normal display
}

//This command is used to specify the display start line address of the DDRAM.
//If the display start line address is changed dynamically using this command, then screen
//scrolling, page swapping can be performed.
void lcd_start_line(unsigned char line)
{
    lcd_write_byte(0x40 | (line & 0x3F), 0);
}

//  (0,0)
void lcd_home(void)
{
	lcd_write_byte(0xB0, 0); // Page address set
    lcd_write_byte(0x10, 0); // Sets the DDRAM column address - upper 3-bit
    lcd_write_byte(0x00, 0); // lower 4-bit
}

//   row
void lcd_set_row(unsigned char row)
{
	lcd_write_byte(0xB0 | (row & 0x0F), 0); // Page address set
}

//   col
void lcd_set_col(unsigned char col)
{
    lcd_write_byte(0x10 | (col>>4), 0); // Sets the DDRAM column address - upper 3-bit
    lcd_write_byte(0x00 | (col & 0x0F), 0); // lower 4-bit
}

/* MCU Hardware depend functions */

#ifdef USE_HW_RESET
//RESET_D 
#define lcd_rst_mark() RESET_D = 0
#define lcd_rst_release() RESET_D = 1
#endif

//CS LCD     
#define lcd_cs_mark()
#define lcd_cs_release()

void lcd_init_gpio(void)
{
#if (USEHW == 1)
	SSPCON = 0b00100000; // SPI Master mode, clock = FOSC/4
	SSPSTAT = 0b00000000;
#endif
        ANSEL = 0;
        TRISA = 0b00100000;
        PORTA = 0;
        ANSELH = 0;
        TRISB = 0;
        PORTB = 0;
        CLK_D = CLK_0;
	TRISC = 0;
        PORTC = 0;
        pLed = 1;
}

void lcd_reset(void)
{
#ifdef USE_HW_RESET
    lcd_rst_mark();
    __delay_ms(10);
    lcd_rst_release();
    __delay_ms(10);
#endif
    lcd_write_byte(0xE2, 0); //RESET
}

#ifdef USE_EEPROM
//Ka  EEPROM.
//   CS=1  .
static void ee_read(unsigned int ee_addr)
{
    MOSI_M = 1; CLK_M = CLK_1;    CLK_M = CLK_0;    // "1"
    CLK_M = CLK_1;    CLK_M = CLK_0;                // "1"
    MOSI_M = 0; CLK_M = CLK_1;    CLK_M = CLK_0;    // "0"
    
#if (USEHW == 1)
    unsigned int d = data;
    SSPDAT = d;
    while(!BF) // Wait while BF == 0
        ;
#else
    unsigned char i;
    for(i=11; i>0; i--)
    {
        if(ee_addr & 0x400) MOSI_M = 1;
                   else MOSI_M = 0;
        ee_addr <<= 1;
        CLK_M = CLK_1;
        __delay_us(CLK_DELAY);
        CLK_M = CLK_0;
    }
#endif
}

// j   EEPROM   ee_addr  
// 93LC86  15-11 ee_addr = 0 !
void eeprom_to_lcd(unsigned int ee_addr, unsigned char j)
{
    unsigned char i;
    MOSI_D = 1; //""
    CS_M = 1;   // EEPROM
    ee_read(ee_addr); // READ
    MOSI_M = 0;

    lcd_cs_mark();
    for(; j>0; j--) // j 
    {
        CLK_D = CLK_1;
        CLK_D = CLK_0;
        TRIS_MOSI_D = 1;    //MOSI_D  
        for(i=8; i>0; i--)  // 8   EEPROM
        {
            CLK_M = CLK_1;
            CLK_M = CLK_0;
            CLK_D = CLK_1;
            CLK_D = CLK_0;
        }
        MOSI_D = 1; TRIS_MOSI_D = 0;    //MOSI_D  , "1"
    }
    lcd_cs_release();
    CS_M = 0;
}
#endif

// /  .
//rc: 0 - , 1 - .
void lcd_write_byte(unsigned char data, unsigned char rc)
{
    lcd_cs_mark();
    if(rc) MOSI_D = 1;
      else MOSI_D = 0;
    CLK_D = CLK_1;
    CLK_D = CLK_0;
    unsigned char i;
    for(i=8; i>0; i--)
    {
        if(data & 0x80) MOSI_D = 1;
                   else MOSI_D = 0;
        data <<= 1;
        CLK_D = CLK_1;
        __delay_us(CLK_DELAY);
        CLK_D = CLK_0;
    }
    lcd_cs_release();
}